# -------------------------------------------------------------------------------
# Copyright IBM Corp. 2017
# 
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
# http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an 'AS IS' BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# -------------------------------------------------------------------------------

from pixiedust.display.chart.renderers import PixiedustRenderer
from pixiedust.utils import Logger
from .brunelBaseDisplay import BrunelBaseDisplay

@PixiedustRenderer(id="scatterPlot")
@Logger()
class ScatterPlotRenderer(BrunelBaseDisplay):

    def compute_brunel_magic(self):
        parts = ["point"]
        valueFields = self.getValueFields()
        keyFields = self.getKeyFields()
        
        parts.append("x({})".format(keyFields[0]))
        if len(valueFields) == 1:
            parts.append("y({})".format(valueFields[0]))
        elif len(valueFields) > 1:
            parts.append("y({})".format(",".join(valueFields)))
            parts.append("color(#series)")
        
        parts.append(self.get_sort())

        return parts
