<div class="form-group" style="margin-left:20px">
    <div class="radio">
        <label><input type="radio" name="refresh" value="1" checked>Never</label>
    </div>
    <div class="radio">
        <label><input type="radio" name="refresh" value="2">Every hour</label>
    </div>
    <div class="radio">
        <label>
            <input type="radio" name="refresh" value="3">On schedule
        </label>
    </div>
</div>