{% macro get_customized_css(class='btn-primary') -%}
{%if this.is_running_on_dsx%}
{%set bg_color='#f0ad4e' if class=='btn-warning' else '#337ab7'%}
{%set border_color='#eea236' if class=='btn-warning' else '#2e6da4'%}
style="padding: 6px 12px;background-color:{{bg_color}};border-color: {{border_color}};margin: 0px;border: 1px solid transparent;"
{%endif%}
{%- endmacro %}

<div class="btn-toolbar" role="toolbar" style="margin:0;">
    <div class="btn-group" role="group" style="margin:0">
        <div class="btn-group" role="group" {{get_customized_css('btn-warning')}}>
            <button type="submit" class="btn btn-warning" style="border-radius: 0;"
                data-original-title="Resume"
                title="Continue execution, only stop when a breakpoint is encountered"
                pd_options="send_input_reply=c"
                pd_target="debugger{{prefix}}">
                <i class="fa fa-play fa-lg"></i>
            </button>
        </div>
        <div class="btn-group" role="group" {{get_customized_css()}}>
            <button type="submit" class="btn btn-primary" style="border-radius: 0;"
                data-original-title="Next"
                title="Continue execution until the next line in the current function is reached or it returns"
                pd_target="debugger_refresh"
                pd_options="send_input_reply={{this.pm_prefix}}n">
                <i class="fa fa-step-forward fa-lg"></i>
            </button>
        </div>
        <div class="btn-group" role="group" {{get_customized_css()}}>
            <button type="submit" class="btn btn-primary" style="border-radius: 0;"
                data-original-title="Step Into"
                title="Execute the current line, stop at the first possible occasion"
                pd_target="debugger_refresh"
                pd_options="send_input_reply={{this.pm_prefix}}s">
                <i class="fa fa-level-down fa-lg"></i>
            </button>
        </div>     
        <div class="btn-group" role="group" {{get_customized_css()}}>
            <button type="submit" class="btn btn-primary" style="border-radius: 0;"
                data-original-title="Step Out"
                title="Continue execution until the current function returns"
                pd_target="debugger_refresh"
                pd_options="send_input_reply={{this.pm_prefix}}r">
                <i class="fa fa-reply fa-lg"></i>
            </button>
        </div>                         
        <div class="btn-group" role="group" {{get_customized_css()}}>
            <button type="submit" class="btn btn-primary" style="border-radius: 0;"
                data-original-title="Up Frame"
                title="Move the current frame one level up in the stack trace"
                pd_target="debugger_refresh"
                pd_options="send_input_reply=u">
                <i class="fa fa-angle-double-up fa-lg"></i>
            </button>
        </div>
        <div class="btn-group" role="group" {{get_customized_css()}}>
            <button type="submit" class="btn btn-primary" style="border-radius: 0;"
                data-original-title="Down Frame"
                title="Move the current frame one level down in the stack trace"
                pd_target="debugger_refresh"
                pd_options="send_input_reply=d">
                <i class="fa fa-angle-double-down fa-lg"></i>
            </button>
        </div>
        <div class="btn-group" style="margin-left:3px" role="group" {{get_customized_css()}}>
            <button type="submit" class="btn btn-primary" style="border-radius: 0;"
                data-original-title="Exit"
                title="Exit the PixieDebugger"
                pd_target="debugger{{prefix}}"
                pd_options="send_input_reply=q">
                <i class="fa fa-square fa-lg"></i>
            </button>
        </div>
    </div>
</div>