<div class="form-group">
    <label for="server{{prefix}}">PixieGateway Server:</label>
    <input type="text" class="form-control" id="server{{prefix}}" name="server" value="{{this.server}}">
    <div class="help-text">
        If you don't have an IP address for a PixieGateway Server  
        <a target="_blank" href="https://medium.com/ibm-watson-data-lab/deploy-your-analytics-as-web-apps-using-pixiedusts-1-1-release-d08067584a14">deploy one</a> 
    </div>
</div>
<div class="form-group">
    <label for="description{{prefix}}">Description:</label>
    <textarea class="form-control" id="description{{prefix}}" name="description" 
        style="height:100px" placeholder="Provide a description for your chart..."/>
</div>

<!--div class="form-group">
    <label for="email{{prefix}}">Email Shared Link:</label>
    <input type="text" class="form-control" id="email{{prefix}}" name="email" 
        placeholder="Enter one or more email addresses separated with comma">

    <label for="message{{prefix}}">Message:</label>
    <textarea class="form-control" id="message{{prefix}}" name="message" 
        style="height:100px" placeholder="I'd like to share this chart with you"/>
</div-->